package com.iden.common.videotool;



import com.iden.common.util.DateUtils;
import org.bytedeco.javacv.*;
import org.bytedeco.javacv.Frame;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.*;


/**
 * 视频工具
 *
 * @author
 */
public class VideoUtil {


    private static final Logger logger = LoggerFactory.getLogger(VideoUtil.class);


    public static Long fetchAllPic(File videoFile, String saveDir, Date shootEndTime) throws Exception {

        Long endTime = shootEndTime.getTime() ;

        java.util.List<File> files = new ArrayList<>();

        FFmpegFrameGrabber ff = new FFmpegFrameGrabber(videoFile);
        ff.start();

        // 此视频时长(ms/毫秒秒)
        Long duration = ff.getLengthInTime() /  1000;
        logger.info("duration==" + duration);

        Long beginTime = endTime - duration;

        // 视频总帧数
        int frameLength = ff.getLengthInFrames();

        logger.info("length==" + frameLength);

        //帧率(1秒多少帧)
        double framRate = ff.getFrameRate();

        int i = 0;
        Frame frame = null;

        while (i < frameLength) {
            frame = ff.grabImage();
            if (frame != null && frame.image != null) {
                Double time = beginTime + (i * (1000/framRate));
                String fileName = DateUtils.formatToDateStr(new Date(time.longValue()),"yyyyMMddHHmmss") + "_" + UUID.randomUUID().toString();
                files.add(writeToFile(frame, saveDir, fileName));
            }
            i++;
        }
        ff.stop();
        return duration;
    }

    private static File writeToFile(Frame frame, String saveDir, String fileName) throws Exception {
        File saveDirFile = new File(saveDir);
        if(!saveDirFile.exists()) {
            saveDirFile.mkdirs();
        }
        File targetFile = new File(saveDir + File.separator + fileName + ".jpg");
        String imgSuffix = "jpg";

        Java2DFrameConverter converter = new Java2DFrameConverter();
        BufferedImage srcBi = converter.getBufferedImage(frame);
        int owidth = srcBi.getWidth();
        int oheight = srcBi.getHeight();
        // 对截取的帧进行等比例缩放
        int width = 800;
        int height = (int) (((double) width / owidth) * oheight);
        BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
        bi.getGraphics().drawImage(srcBi.getScaledInstance(width, height, Image.SCALE_SMOOTH), 0, 0, null);
        try {
            ImageIO.write(bi, imgSuffix, targetFile);
        } catch (Exception e) {
            throw new Exception( "截帧失败");
        }
        return targetFile;
    }



}